//Example.cpp
//Basic use of the stochastic wathershed for image segmentation.
//Requires "CImg.h" available from http://cimg.sourceforge.net/

#include "Graph.h"
#include "MSF.h"
#include "StochasticWS.h"
#include <iostream>
#include <vector>
#include "CImg.h"
#include <algorithm>
#include <math.h>
#include <time.h>
using namespace cimg_library;
using namespace std;

typedef unsigned char  uint8;
typedef CImg<uint8> image;

int main(int argc, char* argv[]){
  
  if(argc!=2){
    cout<<"Usage: a.out <image>"<<endl;
    return 0;
  }

  //Load a grayscale image
  image im=image(argv[1]);
  int W = im.width();
  int H = im.height();
  int nelem=W*H;

  //Build a pixel adjacency graph from the image using 4-connectivity
  //The weight of each edge is taken to be the absolute intensity
  //difference between the pixels spanned by the edge.
  Graph g(nelem);
  int p,q;

  for(int i=0; i<W; i++){
    for(int j=0; j<H; j++){

      if(i+1<W){
	p=im.offset(i,j); 
	q=im.offset(i+1,j);
	g.SetEdge(p,q, abs(im.at(p)-im.at(q)));
      }
      
      if(j+1<H){
	p=im.offset(i,j); 
	q=im.offset(i,j+1);
	g.SetEdge(p,q, abs(im.at(p)-im.at(q)));
      }

    }
  }

  //Calculate minimum spanning forest
  vector<int> roots;
  Graph t= MSF(g,roots);

  //Calculate stochastic watershed PDF (for N=2 seeds)
  Graph ws=StochasticWS(t,2);
  
  //Use the PDF to obtain a final segmentation.
  //In this example, we remove the k most prominent edges and label 
  //the components of the resulting forest.
  vector<Edge> edges=ws.GetEdgesAsVector();

  CompareEdgeByWeightInv Q;
  std::sort(edges.begin(), edges.end(), Q);

  int k=1;
  for(int i=max(0,int(edges.size()-k)); i<edges.size(); i++){ 
      ws.DeleteEdge(edges[i].p, edges[i].q);
  }

  vector<int> labels=ws.GetConnectedComponents();

  //Save result to file
  uint8* d=im.data();

  for(unsigned int i=0; i<nelem; i++){
    d[i]=labels[i];
  }

  im.save_png("result.png");
  
  return 0;
}
