// Exact stochastic watersheds, C++ implementation
// Filip Malmberg 2013.				
// For description, license, and example usage see README.TXT.

#include "Graph.h"

Graph::Graph(int n_nodes){
  size=n_nodes;
  node_edges.resize(size);
}

EdgeSet Graph::GetEdges(){
  EdgeSet edges;
  for(int i=0; i<size; i++){
    auto it=node_edges[i].begin();
    for(it; it!=node_edges[i].end(); it++){
      edges.insert(Edge(i,it->first, it->second));
    }
  }
  return edges;
}

//Returns all unique edges
vector<Edge> Graph::GetEdgesAsVector(){
  vector<Edge> edges;
  
  for(int i=0; i<size; i++){
    auto it=node_edges[i].begin();
    for(it; it!=node_edges[i].end(); it++){
      if(i<it->first){ 
	edges.push_back(Edge(i,it->first, it->second));
      }
    }
  }
  return edges;
}

map<int,double> Graph::GetAdjacent(int p){
    assert(p>=0 && p<size);
    return node_edges[p];
}

void Graph::SetEdge(int p, int q, double w){
  assert(p>=0 && p<size && q>=0 && q<size);
  node_edges[p][q]=w;
  node_edges[q][p]=w;
}

void Graph::DeleteEdge(int p, int q){
  assert(p>=0 && p<size && q>=0 && q<size);
  node_edges[p].erase(q);
  node_edges[q].erase(p);
}

vector<int> Graph::GetConnectedComponents(){
  vector<int> result(size);
  for(int i=0; i<size; i++){
    result[i]=-1;
  }
  
  vector<int> Q;
  
  int label=0;
  for(int i=0; i<size; i++){
    if(result[i]==-1){
      label++;
      Q.push_back(i);
      
      while(!Q.empty()){
	int index=Q.back();
	result[index]=label;
	Q.pop_back();
	auto it=node_edges[index].begin();
	for(it; it!=node_edges[index].end(); it++){
	  if(result[it->first]==-1){
	    Q.push_back(it->first);
	  }
	}
      }
    }
    
    
  }
  
  return result;
}
