// Exact stochastic watersheds, C++ implementation
// Filip Malmberg 2013.				
// For description, license, and example usage see README.TXT.

#include "MSF.h"
#include "Edge.h"
#include <iostream>
#include <queue> 
#include <set>
using namespace std;

//Prim's algorithm
Graph MSF(Graph& g, vector<int> roots){

  if(roots.size()==0){
    roots.push_back(0);
  }

  Graph msf(g.GetNodeCount());

  set<int> visited;
  priority_queue<Edge,vector<Edge>, CompareEdgeByWeight> Q;
  
  map<int, double> neighbors;

  int p,q;
  double w;

  //Initialize
  for(unsigned int i=0; i<roots.size(); i++){
    p=roots[i];
    Q.push(Edge(p,p,0));
  }


  //Front propagation
  while(!Q.empty()){
    Edge e=Q.top();
    Q.pop();

    p=e.q;
    if(visited.find(p)==visited.end()){
      visited.insert(p);
      if(e.p!=e.q){
	msf.SetEdge(e.p, e.q, e.w);
      }
      neighbors=g.GetAdjacent(p);

      auto it=neighbors.begin();
      
      for(it; it!=neighbors.end(); it++){
	q=it->first;
	w=it->second;
	Q.push(Edge(p, q, w));
      }
      
    }
  }

return msf;
}


